package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.common.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrCardInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Description: 通用实名解绑service
 * <br />
 * CreateDate 2022-01-26 10:36:27
 *
 * @author yuyi
 **/
@Service
@Slf4j
public class RnrUnboundServiceImpl implements IRnrUnboundService {

    @Autowired
    private IMgRnrInfoService rnrInfoService;

    @Autowired
    private IMgRnrCardInfoService rnrCardInfoService;

    @Autowired
    private IRnrOrderService orderService;

    @Autowired
    @Lazy
    private IMgRnrCardNoticeService cardNoticeService;

    @Autowired
    private IOrganService organService;

    @Autowired
    private IRnrRelationService relationService;

    @Resource
	private IMgRnrCompanyInfoService companyInfoService;

    /**
     * 通用实名解绑
     *
     * @param bean 参数体
     * @return
     */
    @Override
    @Transactional
    public Response rnrUnbound(RnrUnbindDTO bean) {
        //判断vin号是否为空
        if (StringUtils.isBlank(bean.getIccid())) {
            return Response.createError(ResponseCode.ICCID_NOT_NULL.getMsg(), ResponseCode.ICCID_NOT_NULL.getCode());
        }

        List<MgCardNoticeDTO> noticeDTOS = new ArrayList<>();
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();

        //查询ICCID对应的用户，并判断用户信息和传入的是否一致
        Response response = checkCardAndRnrInfo(bean, cardInfoDTOS, noticeDTOS);
        if (!response.isSuccess()) {
            return response;
        }

        //查询不到需要解绑的信息
        if (noticeDTOS.isEmpty()) {
            return Response.createError(ResponseCode.ICCID_RNR_INFO_NOT_MATCHED.getMsg(), ResponseCode.ICCID_RNR_INFO_NOT_MATCHED.getCode());
        }

        //card表设置成解绑状态
        List<String> uuids = cardInfoDTOS.stream().map(MgRnrCardInfoDTO::getUuid).collect(Collectors.toList());
//        List<String> rnrids = cardInfoDTOS.stream().map(MgRnrCardInfoDTO::getRnrId).collect(Collectors.toList());
        rnrCardInfoService.unboundCardBatch(uuids, RnrStatus.UNBOUND);
//        rnrInfoService.updateRnrStatusBatch(rnrids, RnrStatus.UNBOUND);

        //异步发送解绑消息
        cardNoticeService.sendCardNotice(noticeDTOS);

        return Response.createSuccess();
    }


    @Override
    @Transactional
    public Response batchUnbound(VehicleUnbindDTO bean) {
        log.info("车企实名解绑:{}", JSON.toJSONString(bean));

        //原card表设置成解绑状态
        List<String> cardIds = bean.getCardIds();
        rnrCardInfoService.unboundCardBatch(cardIds, RnrStatus.UNBOUND);
        orderService.updateOrderStatus(Collections.singletonList(bean.getOrder().getUuid()),
                RnrOrderStatusEnum.PASS.getCode());
        //保存实名解绑数据 并发送通知
        Response response = relationService.saveRnrRelation(bean);
        if (!response.isSuccess()) {
            throw new CuscUserException(response.getCode(), response.getMsg());
        }
        return response;
    }

    /**
     * 二手车解绑
     *
     * @param bean
     * @return
     */
    @Override
    public Response secondHandUnBound(RnrRelationDTO bean) {
        return relationService.saveRnrRelation(bean);
    }

    /**
     * 用户信息和传入的一致的加入解绑列表
     */
    private Response checkCardAndRnrInfo(RnrUnbindDTO bean, List<MgRnrCardInfoDTO> cardInfoDTOS, List<MgCardNoticeDTO> noticeDTOS) {
        //根据ICCID查询卡信息
        MgRnrCardInfoDTO parm = new MgRnrCardInfoDTO();
        parm.setIccid(bean.getIccid());
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOS = rnrCardInfoService.getListByIccid(parm);

        for (MgRnrCardInfoDTO cardInfoDTO : mgRnrCardInfoDTOS) {
            //查询实名信息
            String rnrId = cardInfoDTO.getRnrId();
            MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
            mgRnrInfoDTO.setUuid(rnrId);
            mgRnrInfoDTO = rnrInfoService.getByUuid(mgRnrInfoDTO);

            //判断是否加入解绑列表
            //卡是实名状态
            //用户姓名、证件号、证件类型和传入的一致
            if (cardInfoDTO.getRnrStatus() == RnrStatus.RNR.getCode()
                    && StringUtils.equals(mgRnrInfoDTO.getFullName(), bean.getFullName())
                    && StringUtils.equals(mgRnrInfoDTO.getCertNumber(), bean.getCertNumber())
                    && StringUtils.equals(mgRnrInfoDTO.getCertType(), bean.getCertType())) {

                //加入解绑列表
                cardInfoDTOS.add(cardInfoDTO);

                //解绑通知消息
                MgCardNoticeDTO mgCardNoticeDTO = MgRnrCardInfoConverter.INSTANCE.unboundDTOtoNoticeDTO(bean);
                //mgCardNoticeDTO.setOperationType(RnrBizzTypeEnum.Unbound.getCode());
                //mgCardNoticeDTO.setRnrId(mgRnrInfoDTO.getUuid());
                mgCardNoticeDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
                mgCardNoticeDTO.setOrderId(cardInfoDTO.getOrderId());

                //添加项目ID
                //Response<ProjectNumResponseDTO> responseDTOResponse = projectService.findProjectNoByIccid(mgCardNoticeDTO.getIccid());
                //if (!responseDTOResponse.isSuccess()) {
                //    return responseDTOResponse;
                //}
                //mgCardNoticeDTO.setProjectNum(responseDTOResponse.getData().getProjectNum());

                noticeDTOS.add(mgCardNoticeDTO);
            }
        }

        return Response.createSuccess();
    }


	@Override
	public void checkUnBindVehicle(RnrRelationDTO bean) {
		Integer orderType = bean.getOrder().getOrderType();

		if (!orderType.equals(RnrOrderType.NEW_VEHICLE.getCode())
				&& !orderType.equals(RnrOrderType.SEC_VEHICLE.getCode())
				&& !orderType.equals(RnrOrderType.COMPANY_NEW_VEHICLE.getCode())) {
			log.info("checkUnBindVehicle  not orderType orderId = {}",bean.getOrder().getUuid());
			return;
		}

		log.info("checkUnBindVehicle  start orderId = {}",bean.getOrder().getUuid());

		List<String> iccidList = bean.getCardList().stream().map(MgRnrCardInfoDTO::getIccid).distinct().collect(Collectors.toList());
		MgRnrCardInfoDTO cardQuery = new MgRnrCardInfoDTO();
		cardQuery.setIccidList(iccidList);
		List<MgRnrCardInfoDTO> cardInfoDTOList = rnrCardInfoService.queryByList(cardQuery);
		log.info("checkUnBindVehicle 查询到已经卡信息结果 orderId = {},cardInfoDTOList = {}",bean.getOrder().getUuid(),JSONObject.toJSONString(cardInfoDTOList));
		if (CollectionUtils.isEmpty(cardInfoDTOList)) {
			return;
		}

		List<String> rnrOrderIdList = cardInfoDTOList.stream()
				.filter(card -> RnrStatus.RNR.getCode() == card.getRnrStatus())
				.map(MgRnrCardInfoDTO::getOrderId)
				.collect(Collectors.toList());

		if (CollectionUtils.isEmpty(rnrOrderIdList)) {
			return;
		}

		RnrOrderDTO orderQuery = new RnrOrderDTO();
		orderQuery.setUuidList(rnrOrderIdList);
		orderQuery.setOrderType(RnrOrderType.CARMAKER_NEW_VEHICLE.getCode());
		List<RnrOrderDTO> orderDTOList = orderService.queryByList(orderQuery);
		log.info("checkUnBindVehicle 查询车企实名工单信息 orderId = {},orderDTOList = {}",bean.getOrder().getUuid(),JSONObject.toJSONString(orderDTOList));

		if (CollectionUtils.isEmpty(orderDTOList)) {
			return;
		}

		boolean tenantNoMatch = orderDTOList.stream().allMatch(order -> order.getTenantNo().equals(bean.getOrder().getTenantNo()));
		if (!tenantNoMatch) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"租户号不一致，不能覆盖车企实名");
		}

		//车企实名的工单
		List<String> vehicleRnrOrderIdList = orderDTOList.stream().map(RnrOrderDTO::getUuid).collect(Collectors.toList());

		//已经车企实名的卡
		List<MgRnrCardInfoDTO> vehicleRnrCardList = cardInfoDTOList.stream().filter(card -> vehicleRnrOrderIdList.contains(card.getOrderId())).collect(Collectors.toList());
		log.info("checkUnBindVehicle 查询车企实名卡信息 orderId = {},vehicleRnrCardList = {}",bean.getOrder().getUuid(),JSONObject.toJSONString(vehicleRnrCardList));
		if (CollectionUtils.isEmpty(vehicleRnrCardList)){
			return;
		}

		//解绑 已经车企实名的卡
		VehicleUnbindDTO dto = transferToVehicleUnbindDTO(orderDTOList.get(0),vehicleRnrCardList);
		dto.setCardIds(vehicleRnrCardList.stream().map(MgRnrCardInfoDTO::getUuid).distinct().collect(Collectors.toList()));

		this.batchUnbound(dto);
	}


	private VehicleUnbindDTO transferToVehicleUnbindDTO(RnrOrderDTO orderDTO,List<MgRnrCardInfoDTO> bindCardList){
		String rnrId = orderDTO.getRnrId();

		//查询rnrInfo
		MgRnrInfoDTO rnrInfoQuery = new MgRnrInfoDTO();
		rnrInfoQuery.setUuid(rnrId);
		MgRnrInfoDTO rnrInfo = rnrInfoService.getByUuid(rnrInfoQuery);
		log.info("checkUnBindVehicle 查询实名信息单信息 orderId = {},rnrInfo = {}",orderDTO.getUuid(),JSONObject.toJSONString(rnrInfo));


		//查询公司信息
		MgRnrCompanyInfoDTO companyQuery = new MgRnrCompanyInfoDTO();
		companyQuery.setRnrId(rnrId);
		MgRnrCompanyInfoDTO companyInfo = companyInfoService.getByRnrid(companyQuery);
		log.info("checkUnBindVehicle 查询公司信息 orderId = {},companyInfo = {}",orderDTO.getUuid(),JSONObject.toJSONString(companyInfo));

		VehicleUnbindDTO vehicleUnbindDTO = new VehicleUnbindDTO();
		vehicleUnbindDTO.setTenantNo(orderDTO.getTenantNo());
		vehicleUnbindDTO.setIsTrust(0);
		vehicleUnbindDTO.setIsSecondHandCar(0);

		MgRnrInfoDTO vehicleUnbindInfo = new MgRnrInfoDTO();
		BeanUtils.copyProperties(rnrInfo,vehicleUnbindInfo);
		vehicleUnbindInfo.setSerial_number(CuscStringUtils.generateUuid());
		vehicleUnbindInfo.setOperator("");
		vehicleUnbindInfo.setCreator("");
		vehicleUnbindInfo.setUuid(CuscStringUtils.generateUuid());
		vehicleUnbindInfo.setUserId("");
		vehicleUnbindInfo.setIsCompany(1);
		vehicleUnbindInfo.setRnrStatus(RnrStatus.UNBOUND.getCode());
		vehicleUnbindInfo.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
		vehicleUnbindDTO.setInfo(vehicleUnbindInfo);

		RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
		rnrOrderDTO.setOrderType(RnrOrderType.COMPANY_CORPORATION_UNBIND.getCode());
		rnrOrderDTO.setRnrId(vehicleUnbindInfo.getUuid());
		rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
		rnrOrderDTO.setTenantNo(vehicleUnbindInfo.getTenantNo());
		rnrOrderDTO.setOperator("");
		rnrOrderDTO.setCreator("");
		rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.PORTAL.getCode());
		rnrOrderDTO.setSerialNumber(vehicleUnbindInfo.getSerial_number());
		rnrOrderDTO.setIsBatchOrder(0);
		rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
		rnrOrderDTO.setAutoRnr(true);
		rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
		rnrOrderDTO.setSendWorkOrder(false);
		rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
		rnrOrderDTO.setOrgId(vehicleUnbindInfo.getOrgId());
		vehicleUnbindDTO.setOrder(rnrOrderDTO);

		MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO = new MgRnrCompanyInfoDTO();
		BeanUtils.copyProperties(companyInfo,mgRnrCompanyInfoDTO);
		mgRnrCompanyInfoDTO.setUuid(CuscStringUtils.generateUuid());
		mgRnrCompanyInfoDTO.setRnrId(vehicleUnbindInfo.getUuid());
		mgRnrCompanyInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
		mgRnrCompanyInfoDTO.setCreator("");
		mgRnrCompanyInfoDTO.setOperator("");
		//是否是车企
		mgRnrCompanyInfoDTO.setIndustryType("1");
		vehicleUnbindDTO.setCompanyInfo(mgRnrCompanyInfoDTO);

		List<MgRnrCardInfoDTO> cardList = bindCardList.stream().map(card -> {
			MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
			cardInfoDTO.setOldCardId(card.getUuid());
			cardInfoDTO.setUuid(CuscStringUtils.generateUuid());
			cardInfoDTO.setIotId(card.getIotId());
			cardInfoDTO.setIccid(card.getIccid());
			cardInfoDTO.setRnrId(vehicleUnbindInfo.getUuid());
			cardInfoDTO.setRnrStatus(RnrStatus.UNBOUND.getCode());
			cardInfoDTO.setNoticeStatus(NoticeStatusEnum.NONEED.getCode());
			cardInfoDTO.setSerial_number(vehicleUnbindInfo.getSerial_number());
			cardInfoDTO.setOrderId(rnrOrderDTO.getUuid());
			cardInfoDTO.setTenantNo(rnrOrderDTO.getTenantNo());
			cardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
			cardInfoDTO.setCreator("");
			cardInfoDTO.setOperator("");
			return cardInfoDTO;
		}).collect(Collectors.toList());
		vehicleUnbindDTO.setCardList(cardList);

		return vehicleUnbindDTO;
	}
}
